/*
    ParaGUI - crossplatform widgetset
    Copyright (C) 2000  Alexander Pipelka
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
    Alexander Pipelka
    pipelka@teleweb.at
 
    Last Update:      $Author: pipelka $
    Update Date:      $Date: 2000/12/20 23:43:31 $
    Source File:      $Source: /usr/local/CVSROOT/linux/paragui/src/sdlapplication.h,v $
    CVS/RCS Revision: $Revision: 1.49 $
    Status:           $State: Exp $
*/

#ifndef SDLAPPLICATION_H
#define SDLAPPLICATION_H

#ifdef SWIG
%include "swigcommon.h"
%module sdlapplication
%{
#include "sdlapplication.h"
%}
#endif

#include "sdlmessageobject.h"
#include <vector>

#define NO_EVENTTHREAD 1

/**
	@author Alexander Pipelka

	@short The class that represent an application in ParaGUI. It handles the main loop, screen attibutes etc.

        Every Paragui application needs to have a <code>SDLApplication</code> object. The Application object is
        the controller for events, screen initialization, theme support and main loop.
        Here is an example of the steps necesary for creating a simple Paragui application that opens a 640x480
        window with a button in the middle.

        <pre>
        \code		
        #include <paragui.h>
        #include <sdlapplication.h>
        #include <sdlbutton.h>
        
        
        // Event handler for button click 
        PARAGUI_CALLBACK(buttonHandler) { 
          cout << "Ouch !\\n";
          return true;
        } 
          
        
        int main(  int argc, char **argv ) { 
          const int WIDTH  = 640; 
          const int HEIGHT = 480; 
          
          SDLApplication app; 
          
          app.LoadTheme("qnx");                      // Load the QNX theme 
          app.InitScreen(640,480,16,SDL_SWSURFACE);  // SDL_SURFACE | SDL_FULLSCREEN for full screen support

          SDLButton btnOK(NULL,0, SDLWidget::mkrect((WIDTH-100)/2,(HEIGHT-20)/2,100,20),"Push me !");

          btnOK.SetEventCallback(MSG_BUTTONCLICK,buttonHandler); // Set the event handler
          btnOK.Show(); 
           
          app.Run();                                 // Application loop, exit with Esc by default 
        } 

        \endcode
        </pre>
*/

class DECLSPEC SDLApplication : public SDLMessageObject  {
public:

	/**  */
	SDLApplication();

	/**  */
	~SDLApplication();

	/**
		Initialize the screen

		@param	w			screenwidth in pixels
		@param	h			screenheight in pixels
		@param	depth	screendepth in bits per pixel
		@param	flags	SDL screen initialization flags
	*/
#ifdef SWIG
	// swig messes up the default arguments
	bool InitScreen(int w, int h, int depth, unsigned int flags);
#else
	bool InitScreen(int w, int h, int depth=DISPLAY_DEPTH, Uint32 flags = SDL_SWSURFACE /* | SDL_FULLSCREEN*/ | SDL_HWPALETTE);
#endif

	/**
		Load a widget theme

		@param	xmltheme			name of the theme (e.g. default)
		@param	asDefault		apply the theme as default widgettheme
		@param	searchpath		path where the themefile is located
	*/
	SDLTheme* LoadTheme(const char* xmltheme, bool asDefault = true, const char* searchpath = NULL);

	/**
		Run the applications main eventloop
	*/
	SDL_Thread* Run();

	/**
		Exit the main eventloop
	*/
	void Quit();

	/**
		Cleanup the application data
	*/
	void Shutdown();
		
	/**  */
	SDL_Surface* SetScreen(SDL_Surface* screen);

	/**  */
	static SDL_Surface* GetScreen();

	/**  */
	static TTF_Font* GetDefaultFont();

	/**  */
	static bool UnlockScreen();

	/**  */
	static bool LockScreen();

	/**  */
	bool SetBackground(const char* filename, int mode=BKMODE_TILE);

	/**  */
#ifdef SWIG
	%name(SetBackground2) bool SetBackground(SDL_Surface* surface, int mode=BKMODE_TILE);
#else
	bool SetBackground(SDL_Surface* surface, int mode=BKMODE_TILE);
#endif

	/**  */
	void RedrawBackground();

	/**  */
	static void SetApplicationPath(char* path);

	/**  */
	static char* GetApplicationPath();

	/**  */
	static char* GetRelativePath(char* file);

	/**  */
	static int GetScreenHeight();

	/**  */
	static int GetScreenWidth();

	/**  */
	static void FlipPage();

	/**  */
	static TTF_Font* LoadFont(char* filename, int size);

	/**  */
#ifndef SWIG
	void PrintVideoTest();
#endif

	/**  */
	static SDLTheme* GetTheme();

	/**  */
	static bool GetBulkMode();

protected:

	/**  */
	bool eventKeyUp(const SDL_KeyboardEvent* key);

	/**  */
	bool eventQuit(int id, SDLWidget* widget, unsigned long data);

	/**  */
	virtual void eventInit();

private:

	// disable the copy operators
	SDLApplication(const SDLApplication&);
	SDLApplication& operator=(const SDLApplication&);

	bool my_freeBackground;
	SDL_Surface* my_background;
	int my_backmode;

	static bool FileExists(const char* filename);

	static SDLTheme* my_Theme;

#ifndef SWIG
	static char app_path[300];
#endif

	static SDLApplication* pGlobalApp;
	static SDL_Surface* screen;
	static TTF_Font* font;
	static SDL_mutex* mutexScreen;
	static bool bulkMode;
};

#endif // SDLAPPLICATION_H
