/*
    ParaGUI - crossplatform widgetset
    Copyright (C) 2000  Alexander Pipelka
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
    Alexander Pipelka
    pipelka@teleweb.at
 
    Last Update:      $Author: pipelka $
    Update Date:      $Date: 2000/12/20 17:20:08 $
    Source File:      $Source: /usr/local/CVSROOT/linux/paragui/src/sdldrawobject.h,v $
    CVS/RCS Revision: $Revision: 1.38 $
    Status:           $State: Exp $
*/

#ifndef SDLDRAWOBJECT_H
#define SDLDRAWOBJECT_H

#ifdef SWIG
%include "swigcommon.h"
%module sdldrawobject
%{
#include "sdldrawobject.h"
%}
#endif

#include "paragui.h"

/**
	@author Alexander Pipelka
	
	@short Base class of all objects (provides drawing functionality).
 
	Provides a graphics context and basic drawing functionality.
*/

class DECLSPEC SDLDrawObject {
public:
	
	/** */
	SDLDrawObject(int id, SDL_Rect& rect, bool visible, bool storeBackground, SDLDrawObject* objParent = NULL);

	/** */
	virtual ~SDLDrawObject();

	// draw, update, move, ...

	/**
	Redraw the surface of the widget

	@param	update				if true SDLDrawObject::Update(true) is called after the redraw
	@return							redraw succeeded
	*/
	virtual bool Redraw(bool update = true);

	/**
	Update the widgets screen area

	@param	doBlit				if true SDLDrawObject::Blit() is called before the update
	*/
	virtual void Update(bool doBlit = true);

	/**
	Blit the surface of the object to the screen surface
	*/
	virtual void Blit();

	/**
	Move the object to a given screen-coordinate

	@param	x						screen x-position
	@param	y						screen y-position
	@return							move succeeded
	*/
	virtual bool MoveWindow(int x, int y);

	/**
	Resize the object

	@param	w						new width in pixels
	@param	h						new height in pixels
	@return							resize succeeded
	*/
	virtual bool SizeWindow(int w, int h);

	// set object props

	/**
	Make an object visible

	@param	visible				show / hide the object
	@return							success
	*/
	bool SetVisible(bool visible);

	/**
	Check if the object is visible
	@return							true if object is visible
	*/
	bool IsVisible();

	/**  */
	void SetTransparency(Uint8 t);

	/**  */
	Uint8 GetTransparency();

	/**  */
	bool RestoreBackground();

	/**  */
	bool StoreBackground();

	/**  */
	void SetStoreBackground(bool store);

	/**  */
	bool GetStoreBackground();
	
	/**  */
	virtual void SetScreenUpdate(bool update);

	/**  */
	bool GetScreenUpdate();

	/**  */
	void FadeOut();

	/**  */
	void FadeIn();

	// Drawing functions

	/**  */
	static void SetPixel(int x, int y, Uint8 r, Uint8 g, Uint8 b, SDL_Surface* surface);

	/**  */
	void DrawHLine(int x, int y, int w, Uint8 r, Uint8 g, Uint8 b, SDL_Surface* surface);

	/**  */
	void DrawVLine(int x, int y, int h, Uint8 r, Uint8 g, Uint8 b, SDL_Surface* surface);

	/**  */
	void DrawRectWH(int x, int y, int w, int h, Uint8 r, Uint8 g, Uint8 b, SDL_Surface* surface);

	/** */
	void DrawBorder(SDL_Surface* surface, SDL_Rect* r, int size, bool up = true);

	/**
	Creates a surface filled with a gradient

	@param r		the dimensions of the surface to be created
	@param gradient	the gradient definition
	@return			a SDL_Surface pointer to the new surface

	This function creates a new surface filled with a given gradient
	*/
	static SDL_Surface* CreateGradient(SDL_Rect& r, SDL_Gradient& gradient);

	/**
	Creates a surface filled with a gradient

	@param r		the dimensions of the surface to be created
	@param ul		upper/left gradient color
	@param ur		upper/right gradient color
	@param dl		lower/left gradient color
	@param dr		lower/right gradient color
	@return			a SDL_Surface pointer to the new surface

	This function creates a new surface filled with a given gradient defined by a set of colors
	*/
#ifdef SWIG
	%name(CreateGradient2) static SDL_Surface* CreateGradient(SDL_Rect& r, SDL_Color& ul, SDL_Color& ur, SDL_Color& dl, SDL_Color& dr);
#else
	static SDL_Surface* CreateGradient(SDL_Rect& r, SDL_Color& ul, SDL_Color& ur, SDL_Color& dl, SDL_Color& dr);
#endif

	/**  */
#ifdef SWIG
	%name(DrawGradient2) static void DrawGradient(SDL_Surface* surface, SDL_Rect& r, SDL_Color& ul, SDL_Color& ur, SDL_Color& dl, SDL_Color& dr);
#else
	static void DrawGradient(SDL_Surface* surface, SDL_Rect& r, SDL_Color& ul, SDL_Color& ur, SDL_Color& dl, SDL_Color& dr);
#endif

	/**  */
	static void DrawGradient(SDL_Surface* surface, SDL_Rect& r, SDL_Gradient& gradient);

	/**
	Draw a 'themed' surface

	@param surface		the surface to draw on
	@param r			the rectangle of the surface to draw in
	@param gradient		pointer to a gradient structure (may be NULL)
	@param background	pointer to a background surface (may be NULL)
	@param bkmode		the mode how to fill in the background surface (BKMODE_TILE | BKMODE_STRETCH)
	@param blend		the blend-level between gradient an background
	*/
	static void DrawThemedSurface(SDL_Surface* surface, SDL_Rect* r, SDL_Gradient* gradient, SDL_Surface* background, int bkmode, Uint8 blend);

	/**
	Create a 'themed' surface

	@param r			the dimensions of the surface to create
	@param gradient		pointer to a gradient structure (may be NULL)
	@param background	pointer to a background surface (may be NULL)
	@param bkmode		the mode how to fill in the background surface (BKMODE_TILE | BKMODE_STRETCH)
	@param blend		the blend-level between gradient an background
	@return 			the newly created surface
	*/
	static SDL_Surface* CreateThemedSurface(SDL_Rect& r, SDL_Gradient* gradient, SDL_Surface* background, int bkmode, Uint8 blend);

	/**  */
	void DrawText(int x, int y, char* text, SDL_Color c, TTF_Font* textfont = NULL);

	/**  */
#ifdef SWIG
	%name(DrawText2) void DrawText(SDL_Rect* r, char* text, SDL_Color c, int align = SDL_TA_LEFT, TTF_Font* textfont = NULL);
#else
	void DrawText(SDL_Rect* r, char* text, SDL_Color c, int align = SDL_TA_LEFT, TTF_Font* textfont = NULL);
#endif

	// clipping / intersection

	/**  */
	static SDL_Rect IntersectRect(SDL_Rect p, SDL_Rect c);

	/**  */
	void SetClipRect(int x, int y, int w, int h);

	/**  */
	bool IsClippingEnabled();

	/**  */
	void GetClipRects(SDL_Rect& src, SDL_Rect& dst, SDL_Rect* displayrect = NULL);

	/** */
	SDL_Rect GetClipRect();

	/**  */
	virtual bool IsDisplayRectValid();

	/** */
	void SetParent(SDLDrawObject* objParent);

protected:

	/**  */
	SDL_Surface* CreateSubSurface(SDL_Surface* surface, SDL_Rect* r);

	/**  */
	virtual void eventDraw(SDL_Surface* surface, SDL_Rect* rect);

	/**  */
	virtual void eventBlit(SDL_Surface* srf, SDL_Rect* src, SDL_Rect* dst);

	/**  */
	virtual void eventSizeWindow(int w, int h);

	/**  */
	virtual void eventMoveWindow(int x, int y);

	int my_id;

	SDL_Rect my_rectDisplay;
	SDL_Rect my_rectClip;

	bool my_visible;
	bool my_storeBackground;

	Uint8 my_transparency;

	SDL_Surface* my_srfBackground;
	SDL_Surface* my_srfObject;
	SDL_Surface* my_srfScreen;
	SDL_Surface* my_srfDrag;

	SDLDrawObject* my_objParent;
	SDL_Point my_ptDragStart;

private:
	bool my_screenUpdate;

private: // disable the copy operators
	SDLDrawObject(const SDLDrawObject&);
	SDLDrawObject& operator=(const SDLDrawObject&);
};

#endif // SDLDRAWOBJECT_H
