/*
    ParaGUI - crossplatform widgetset
    Copyright (C) 2000  Alexander Pipelka
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
    Alexander Pipelka
    pipelka@teleweb.at
 
    Last Update:      $Author: pipelka $
    Update Date:      $Date: 2001/01/12 13:18:29 $
    Source File:      $Source: /usr/local/CVSROOT/linux/paragui/src/sdlmessageobject.h,v $
    CVS/RCS Revision: $Revision: 1.44 $
    Status:           $State: Exp $
*/

#ifndef SDLMESSAGEOBJECT_H
#define SDLMESSAGEOBJECT_H

#ifdef SWIG
%include "swigcommon.h"
%module sdlmessageobject
%{
#include "sdlmessageobject.h"
%}
#endif

#include "paragui.h"

//class SDLMessageObject;
class SDLWidget;

/**
	@author Alexander Pipelka
	
	@short Base class of all objects (provides message handling).
 
	Provides a message pump and global handlers for all other SDLMessageObject instances.
*/

class DECLSPEC SDLMessageObject {

public:
	/**
	Creates a SDLMessageObject

	An application must have a maximum of one SDLMessageObject. If you try to create more than one SDLMessageObject
	the constructor will exit your application with an console error message.
	*/
	SDLMessageObject();

	/**
	Destroys a SDLMessageObject and removes it from the global object list.
	*/
	virtual ~SDLMessageObject();

	/**
	This function enables or disables receiving of SDL_Event messages.

	@param enable	Enable or disable the ability of receiving messages. When set to false
			no event handlers will be called for this object

	*/
	void EnableReceiver(bool enable);

	/**
	Set a message capture for this object. This object will receive all SDL_Event messages regardless
	if it is able to process them or not.
	*/
	SDLMessageObject* SetCapture();

	/**
	Releases a previous capture.
	*/
	void ReleaseCapture();

	/** */
	SDLMessageObject* SetInputFocus();

	/** */
	void ReleaseInputFocus();

	/**
	Run the modal message pump. This function will exit when the main window was closed.
	*/
	int RunEventLoop(void* data);

	/**
	returns true when the object can receive messages or false if not.
	*/
	bool IsEnabled();

	/**  */
	bool SendMessage(SDLMessageObject* target, MSG_TYPE type, MSG_ID id, MSG_DATA data);

	/** */
	void SetEventCallback(MSG_TYPE type, MSG_CALLBACK cbfunc, void *clientdata = NULL);
	
	/**
	Sends an event to the global message queue.

	@param event SDL_Event message

	@return Notifies the message pump if this message is processed by this object or it should be routed to the next message receiver.
	*/

	static int PumpIntoEventQueue(const SDL_Event* event);

	/** */
	SDL_Event WaitEvent(Uint32 delay=0);

protected:

	/**
	Overridable Eventhandler for the SDL_ActiveEvent message.
	The default implementation returns 'false' which indicates that this message is not processed by this object.

	@param active SDL_ActiveEvent message

	@return Notifies the message pump if this message is processed by this object or it should be routed to the next message receiver.
	*/
	virtual bool eventActive(const SDL_ActiveEvent* active);

	/**
	Overridable Eventhandler for a SDL_KeyboardEvent message.
	This handler is called when a key changed it's state from unpressed to pressed.
	The default implementation returns 'false' which indicates that this message is not processed by this object.

	@param key SDL_KeyboardEvent message

	@return Notifies the message pump if this message is processed by this object or it should be routed to the next message receiver.
	*/
	virtual bool eventKeyDown(const SDL_KeyboardEvent* key);

	/**
	Overridable Eventhandler for a SDL_KeyboardEvent message.
	This handler is called when a key changed it's state from pressed to unpressed.
	The default implementation returns 'false' which indicates that this message is not processed by this object.

	@param key SDL_KeyboardEvent message

	@return Notifies the message pump if this message is processed by this object or it should be routed to the next message receiver.
	*/
	virtual bool eventKeyUp(const SDL_KeyboardEvent* key);

	/**
	Overridable Eventhandler for a SDL_MouseMotionEvent message.
	This handler is called when mouse movement is detected.
	The default implementation returns 'false' which indicates that this message is not processed by this object.

	@param motion SDL_MouseMotionEvent message

	@return Notifies the message pump if this message is processed by this object or it should be routed to the next message receiver.
	*/
	virtual bool eventMouseMotion(const SDL_MouseMotionEvent* motion);

	/**
	Overridable Eventhandler for a SDL_MouseButtonEvent message.
	This handler is called when a mouse button is pressed.
	The default implementation returns 'false' which indicates that this message is not processed by this object.

	@param button SDL_MouseButtonEvent message

	@return Notifies the message pump if this message is processed by this object or it should be routed to the next message receiver.
	*/
	virtual bool eventMouseButtonDown(const SDL_MouseButtonEvent* button);

	/**
	Overridable Eventhandler for a SDL_MouseButtonEvent message.
	This handler is called when a mouse button is released.
	The default implementation returns 'false' which indicates that this message is not processed by this object.

	@param button SDL_MouseButtonEvent message

	@return Notifies the message pump if this message is processed by this object or it should be routed to the next message receiver.
	*/
	virtual bool eventMouseButtonUp(const SDL_MouseButtonEvent* button);

	/**
	Overridable Eventhandler for a SDL_QuitEvent message.
	The default implementation returns 'false' which indicates that this message is not processed by this object.

	@param quit SDL_QuitEvent message

	@return Notifies the message pump if this message is processed by this object or it should be routed to the next message receiver.
	*/
	virtual bool eventQuit(int id, SDLWidget* widget, unsigned long data);

	/**
	Overridable Eventhandler for a SDL_SysWMEvent message.
	The default implementation returns 'false' which indicates that this message is not processed by this object.

	@param syswm SDL_SysWMEvent message

	@return Notifies the message pump if this message is processed by this object or it should be routed to the next message receiver.
	*/
	virtual bool eventSysWM(const SDL_SysWMEvent* syswm);

	/**
	Overridable Eventhandler for a SDL_SysUserEvent message.
	The default implementation returns 'false' which indicates that this message is not processed by this object.

	@param event SDL_SysUserEvent message

	@return Notifies the message pump if this message is processed by this object or it should be routed to the next message receiver.
	*/
	virtual bool eventMessage(MSG_MESSAGE* msg);

	/**  */
	virtual bool eventButtonClick(int id, SDLWidget* widget);

	/**  */
	virtual bool eventScrollPos(int id, SDLWidget* widget, unsigned long data);

	/**  */
	virtual bool eventScrollTrack(int id, SDLWidget* widget, unsigned long data);

	/** */
	virtual void eventInputFocusLost(SDLMessageObject* newfocus);

	/** */
	virtual void eventIdle();
	
	/**
	Overridable message filter function. Derivated classes can filter special events.
	The default implementation returns 'false' which indicates that this message is not processed by this object.

	@param event SDL_Event message

	@return Notifies the message pump if this message is processed by this object or it should be routed to the next message receiver.
	*/
	virtual bool AcceptEvent(const SDL_Event* event);

	/**
	Sends an event directly to an object.

	@param event SDL_Event message

	@return Notifies the message pump if this message is processed by this object or it should be routed to the next message receiver.
	*/
	virtual bool ProcessEvent(const SDL_Event* event);

	bool my_quitEventLoop;

private:

	SDLMessageObject(const SDLMessageObject&);
	SDLMessageObject& operator=(const SDLMessageObject&);

	static void SetObjectList(SDLMessageObject* list);
	void RemoveObject(SDLMessageObject* obj);

	static SDLMessageObject* objectList;
	static SDLMessageObject* captureObject;
	static SDLMessageObject* inputFocusObject;

	SDLMessageObject* my_oldCapture;
	SDLMessageObject* my_oldFocus;
	SDLMessageObject* my_objectNext;

	bool my_canReceiveMessages;

	// mutexes

	SDL_mutex* my_mutexSendMessage;
	SDL_mutex* my_mutexReceiveMessage;
	
	// callback functions
	
	/*
	MSG_CALLBACK my_cbFunc[MSG_TABLE_SIZE];
	void* my_cbData[MSG_TABLE_SIZE];
	*/
};


#ifdef SWIG

// Grab a Python function object as a Python object.
%typemap(python,in) PyObject *pyfunc {
   if (!PyCallable_Check($source)) {
      PyErr_SetString(PyExc_TypeError, "Need a callable object!");
      return NULL;
   }
   $target = $source;
}


%{

   /* This function matches the prototype of the normal C callback
      function for our widget. However, we use the clientdata pointer
      for holding a reference to a Python callable object. */
static bool PythonCallBack(int id, SDLWidget *widget, unsigned long data, void *clientdata)
{
   PyObject *func, *arglist;
   PyObject *result;
   bool     res = 0;
   
   func = (PyObject *) clientdata;               // Get Python function
   arglist = Py_BuildValue("(ill)",id,widget,data);// Build argument list
   result = PyEval_CallObject(func,arglist);     // Call Python
   Py_DECREF(arglist);                           // Trash arglist
   if (result) {                                 // If no errors, return double
      res = PyInt_AsLong(result);
   }
   Py_XDECREF(result);
   return res;
}

// Attach a new method to our plot widget for adding Python functions
// Set a Python function object as a callback function
// Note : PyObject *pyfunc is remapped with a typempap
%}

%addmethods SDLMessageObject {
   void set_pymethod(MSG_TYPE type, PyObject *pyfunc) {
      self->SetEventCallback(type, PythonCallBack, (void *) pyfunc);
      Py_INCREF(pyfunc);
   }
}
#endif

#endif // SDLMESSAGEOBJECT_H
