/*
    ParaGUI - crossplatform widgetset
    Copyright (C) 2000  Alexander Pipelka
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
    Alexander Pipelka
    pipelka@teleweb.at
 
    Last Update:      $Author: pipelka $
    Update Date:      $Date: 2001/01/17 20:02:39 $
    Source File:      $Source: /usr/local/CVSROOT/linux/paragui/src/sdlwidget.h,v $
    CVS/RCS Revision: $Revision: 1.55 $
    Status:           $State: Exp $
*/

#ifndef SDLWIDGET_H
#define SDLWIDGET_H

#ifdef SWIG
%include "swigcommon.h"
%module sdlwidget
%{
#include "sdlwidget.h"
%}
#endif

#include "sdlmessageobject.h"
#include "sdldrawobject.h"

/**
	@author Alexander Pipelka
	
	@short Base class of all widgets.
 
	Provides graphics context and message handling functionality.
*/

class DECLSPEC SDLWidget : public SDLMessageObject, public SDLDrawObject  {
public:
	/**
	Creates a SDLWidget

	@param parent	the parentobject for the new widget or NULL if it is a toplevel widget
	@param rect	initial position for the widget
	@param storebackground	flag for backing store

	This is the constructor for the SDLWidget class (really!)
	*/
	SDLWidget(SDLWidget* parent, SDL_Rect& rect, bool storeBackground = false);

	/**
	Destroys a SDLWidget

	This is the destructor for the SDLWidget class
	*/
	~SDLWidget();

	/**
	Load a style from the theme definition

	@param		widgettype		name of the widgettype
	@param		objectname		name of the object

	Loads the defined style for a given widgettype
	*/
	virtual void LoadThemeStyle(const char* widgettype);

	/**
	Load a style from the theme definition

	@param		widgettype		name of the widgettype
	@param		objectname		name of the object

	Loads the defined style of a given widgettype and objectname.
	*/
	virtual void LoadThemeStyle(const char* widgettype, const char* objectname);

	/**
	Start to drag a widget

	StartWidgetDrag prepares to widget to be dragged.
	*/
	void StartWidgetDrag();

	/**  */
	void WidgetDrag(int x, int y);

	/**  */
	void EndWidgetDrag(int x, int y);

	/**
	Move a widget

	@param x	new x-position (in parent context)
	@param y	new y-position (in parent context)
	@return		function succeeded

	This function moves the widget
	*/
	bool MoveWindow(int x, int y);

	/**
	Resize a widget

	@param w	new widget-width
	@param w	new widget-height
	@return		function succeeded

	This function resizes the widget
	*/
	bool SizeWindow(int w, int h);

	/**  */
	SDL_Point ClientToScreen(int sx, int sy);

	/**  */
	SDL_Point ScreenToClient(int x, int y);

	/**  */
	int Width();

	/**  */
	int Height();

	/**  */
	SDL_Surface* GetWidgetSurface();

	/**  */
	SDL_Rect GetWidgetRect();

	/**  */
	SDL_Surface* GetScreenSurface();

	/**
	Get the parentwidget of a widget
	
	@return		a pointer to the parentwidget or NULL if there is no parentwidget
	*/
	SDLWidget* GetParent();

	/**  */
	void AddChild(SDLWidget* child);

	/**
	Process a native SDL event

	@param	event	SDL event to process
	@return			true if the event was sucessfully processed

	ProcessEvent asks the widget to process a given event.
	*/
	bool ProcessEvent(const SDL_Event* event);

	/**  */
	void SetID(int id);

	/**  */
	int GetID();

	/**  */
	bool IsInside(SDL_Point p, SDL_Rect* r = NULL);

	/**  */
	bool IsMouseInside();

	/**  */
	bool IsDisplayRectValid();

	/**  */
	bool Redraw(bool update = true);

	/**  */
	void Hide(bool fade = false);

	/**  */
	void Show(bool fade = false);

	/**  */
	void Update(bool doBlit = true);

	/**  */
	void Blit(bool recursive = true);

	/** */
	void SetChildTransparency(Uint8 t);

	/**
	Get the number of childwidgets

	@return		number of childwidgets
	*/
	int GetChildCount();

	/**
	Set the font

	@param newfont	the widget's new font

	This function sets the font of the widget
	*/
	void SetFont(TTF_Font* newfont);

	/** Returns the current font */
	TTF_Font* GetFont();

	/**
	OBSOLETE
	*/
	void SetScreenUpdate(bool update);

	/**  */
	void MoveRect(int x, int y);

	/**  */
	static SDL_Rect& mkrect(int x, int y, int w, int h);

	/** */
	void RecalcClipRect();

	/** */
	void SetTextColor(Uint32 color);

	/** */
#ifdef SWIG
	/** SWIG can't handle multiple functions with one name */
	%name(SetTextColor2) void SetTextColor(SDL_Color c);
#else
	void SetTextColor(SDL_Color c);
#endif
	
	/** */
	SDL_Color GetTextColor();
	
	/** */
	static void BulkUpdate();
		
protected:

	/**  */
	virtual void eventMouseLeave();

	/**  */
	virtual void eventMouseEnter();

	/**  */
	virtual void eventShow();

	/**  */
	virtual void eventHide();

	/**  */
	bool AcceptEvent(const SDL_Event* event);

	/**	*/
	void RemoveChild(SDLWidget* child);

	bool bSetCaptureOnShow;

	static SDLWidget* widgetList;
	SDLWidget* my_widgetListNext;
	SDLWidget* my_widgetListPrev;

private:

	SDLWidget(const SDLWidget&);
	SDLWidget& operator=(const SDLWidget&);

	SDLWidget* my_widgetParent;
	SDLWidget* my_childList;
	SDLWidget* my_childNext;

	SDL_mutex* my_mutexProcess;
	TTF_Font* my_font;
	SDL_Color my_textcolor;
	
	int my_childCount;
	bool my_mouseInside;
	int my_zdepth;
	bool my_haveTooltip;

	static bool bBulkUpdate;
};

#endif // SDLWIDGET_H
